//
//  ViewController.swift
//  Gesture
//
//  Created by swift on 2019/12/26.
//  Copyright © 2019 kydqitb. All rights reserved.
//

import UIKit

class ViewController: UIViewController {

    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view.
        addTitleLabel()
        
        let aView = UIView()
        aView.backgroundColor = UIColor.red
        view.addSubview(aView)
        addAutoLayout(obj: aView)
        
        addGesture(aView: aView)
    }

    func addTitleLabel() {
        let title = UILabel(frame: CGRect(x: 0, y: 40, width: self.view.frame.width, height: 40))
        title.textAlignment = .center
        title.font = UIFont.systemFont(ofSize: 30)
        title.text = "UIGestureRecognizer基础使用"
        self.view.addSubview(title)
    }
    
    func addGesture(aView:UIView) {
        // 1. 点击手势
        // 单击方法
        let tapOne = UITapGestureRecognizer(target: self, action: #selector(clickWithTapOne))
        aView.addGestureRecognizer(tapOne)
        
        //双击方法
        let tapDouble = UITapGestureRecognizer(target: self, action: #selector(clickWithtapDouble))
        tapDouble.numberOfTapsRequired = 2  //设置双击数， 也可以设置3连击、4连击等等
        aView.addGestureRecognizer(tapDouble)
        
        //声明单击事件需要双击事件检测失败后才会执行，不设置此项双击时也会触发单击
        tapOne.require(toFail: tapDouble)
        
        //设置点击手指的个数
        let touchNum = UITapGestureRecognizer(target: self, action: #selector(clickWithTouchNum(gesture:)))
        touchNum.numberOfTouchesRequired = 2  //要求两个手指进行点击
        aView.addGestureRecognizer(touchNum)
        
        // 2. 拖动手势
        // 拖动过程会支持触发didPan(gesture:)函数
        let pan = UIPanGestureRecognizer(target: self, action: #selector(didPan(gesture:)))
        pan.maximumNumberOfTouches = 2 //设置最多手指
        aView.addGestureRecognizer(pan)
        
        // 3. 长按手势
        let longPress = UILongPressGestureRecognizer(target: self, action: #selector(didLongPress(gesture:)))
        longPress.minimumPressDuration = 1 //设置最小的按压时间， 默认是0.5秒触发didLongPress(gesture:)， 设置为1 是1秒之后触发
        aView.addGestureRecognizer(longPress)
        
        // 4. 捏合手势
        let pinch = UIPinchGestureRecognizer(target: self, action: #selector(didPinch(gesture:)))
        aView.addGestureRecognizer(pinch)
        
        // 5. 旋转手势
        let rotation = UIRotationGestureRecognizer(target: self, action: #selector(didRotation(gesture:)))
        aView.addGestureRecognizer(rotation)
        
        
        
        // 6. 边缘滑动
        let edgePan = UIScreenEdgePanGestureRecognizer(target:self, action:#selector(didEdgePan(gesture:)))
        edgePan.edges = UIRectEdge.right //从右边缘开始滑动
        self.view.addGestureRecognizer(edgePan) //我添加到了self.view，更好看现象
        
        // 7. 滑动手势
        // 向上滑动
        let swipeUp = UISwipeGestureRecognizer(target: self, action: #selector(didSwipeGesture(gesture:)))
        swipeUp.direction = .up //设置滑动的方向
        self.view.addGestureRecognizer(swipeUp)
        
        // 向下滑动
        let swipeDown = UISwipeGestureRecognizer(target: self, action: #selector(didSwipeGesture(gesture:)))
        swipeDown.direction = .down
        self.view.addGestureRecognizer(swipeDown)
        
        // 向左滑动
        let swipeLeft = UISwipeGestureRecognizer(target: self, action: #selector(didSwipeGesture(gesture:)))
        swipeLeft.direction = .left
        self.view.addGestureRecognizer(swipeLeft)
        
        // 向右滑动
        let swipeRight = UISwipeGestureRecognizer(target: self, action: #selector(didSwipeGesture(gesture:)))
        swipeRight.direction = .right
        self.view.addGestureRecognizer(swipeRight)
        
    }
    
    @objc func clickWithTapOne() {
        print("单击了view")
    }
    
    @objc func clickWithtapDouble() {
        print("双击了view")
    }
    
    @objc func clickWithTouchNum(gesture: UITapGestureRecognizer) {
        print("你使用了%d点击了view", gesture.numberOfTouchesRequired)
    }
    
    @objc func didPan(gesture: UIPanGestureRecognizer) {
        print("拖动了view")
    }
    
    @objc func didLongPress(gesture: UILongPressGestureRecognizer) {
        print("你进行了长按view动作")
    }
    
    @objc func didPinch(gesture: UIPinchGestureRecognizer) {
        print("你进行了捏合view手势")
        print("捏合比例:%@", gesture.scale)
        print("捏合速度:%d", gesture.velocity)
    }
    
    @objc func didRotation(gesture: UIRotationGestureRecognizer) {
        print("你进行了旋转view手势")
        print("旋转的角度:%@", gesture.rotation*(180/(CGFloat(Double.pi))) )
    }
    
    @objc func didEdgePan(gesture: UIScreenEdgePanGestureRecognizer) {
        print("你进行了边缘滑动 %@", gesture.edges)
    }
    
    @objc func didSwipeGesture(gesture: UISwipeGestureRecognizer) {
        switch gesture.direction
        {
        case UISwipeGestureRecognizer.Direction.up:
            print("向上滑动")
        case UISwipeGestureRecognizer.Direction.down:
            print("向下滑动")
        case UISwipeGestureRecognizer.Direction.left:
            print("向左滑动")
        case UISwipeGestureRecognizer.Direction.right:
            print("向右滑动")
        default:
            print("其它滑动")
        }
    }
    
    //AutoLayout 自动布局
    func addAutoLayout(obj: UIView) {
        //除了AutoLayout，AutoresizingMask也是一种布局方式。当我们用AutoLayout代码添加视图时默认情况下，translatesAutoresizingMaskIntoConstraints ＝ true , 此时视图的AutoresizingMask会被转换成对应效果的约束。这样很可能就会和我们手动添加的其它约束有冲突。此属性设置成false时，AutoresizingMask就不会变成约束。也就是说当前视图的 AutoresizingMask失效了。 避免布局冲突，把此设置为false
        obj.translatesAutoresizingMaskIntoConstraints = false
        
        //        自动布局构造函数
        //        NSLayoutConstraint(item: 视图,
        //                           attribute: 约束属性,
        //                           relatedBy: 约束关系,
        //                           toItem: 参照视图,
        //                           attribute: 参照属性,
        //                           multiplier: 乘积,
        //                           constant: 约束数值)
        //设置label控件的左边到self.view的左边距离为50pt
        let objLeft:NSLayoutConstraint = NSLayoutConstraint(item: obj, attribute: NSLayoutConstraint.Attribute.left, relatedBy: NSLayoutConstraint.Relation.equal, toItem: self.view, attribute: NSLayoutConstraint.Attribute.left, multiplier: 1.0, constant: 50)
        obj.superview!.addConstraint(objLeft)
        
        //设置label控件的顶部到self.view的顶部距离为100pt
        let objTop:NSLayoutConstraint = NSLayoutConstraint(item: obj, attribute: NSLayoutConstraint.Attribute.top, relatedBy: NSLayoutConstraint.Relation.equal, toItem: self.view, attribute: NSLayoutConstraint.Attribute.top, multiplier: 1.0, constant: 100)
        obj.superview!.addConstraint(objTop)
        
        //设置label控件的右边到self.view的右边距离为50pt
        let objRight:NSLayoutConstraint = NSLayoutConstraint(item: obj, attribute: NSLayoutConstraint.Attribute.right, relatedBy: NSLayoutConstraint.Relation.equal, toItem: self.view, attribute: NSLayoutConstraint.Attribute.right, multiplier: 1.0, constant: -50)
        obj.superview!.addConstraint(objRight)
        
        //设置label控件的高度为50pt
        let objHeight:NSLayoutConstraint = NSLayoutConstraint(item: obj, attribute: NSLayoutConstraint.Attribute.height, relatedBy: NSLayoutConstraint.Relation.equal, toItem: nil, attribute:NSLayoutConstraint.Attribute.notAnAttribute, multiplier: 1.0, constant: 100)
        obj.superview!.addConstraint(objHeight)
    }

}




